@@ -1,34 +1,38 @@
-/* ETable widget - utilities for drawing table header buttons
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-table-header-utils.h
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Copyright (C) 2000 Ximian, Inc.
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@ximian.com>
+ *   Federico Mena-Quintero <federico@ximian.com>
  *
- * Authors: Chris Lahey <clahey@ximian.com>
- *          Miguel de Icaza <miguel@ximian.com>
- *          Federico Mena-Quintero <federico@ximian.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include "e-table-header-utils.h"
+
 #include <string.h> /* strlen() */
 #include <glib.h>
 #include "e-table-defines.h"
-#include "e-table-header-utils.h"
+#include <gal/widgets/e-unicode.h>
 
 
 
@@ -190,6 +194,8 @@ make_composite_pixmap (GdkDrawable *drawable, GdkGC *gc,
 	return pixmap;
 }
 
+static GtkWidget *g_label;
+
 /**
  * e_table_header_draw_button:
  * @drawable: Destination drawable.
@@ -212,7 +218,7 @@ make_composite_pixmap (GdkDrawable *drawable, GdkGC *gc,
 void
 e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 			    GtkStyle *style, GdkFont *font, GtkStateType state,
-			    GtkWidget *widget, GdkGC *gc,
+			    GtkWidget *widget,
 			    int x, int y, int width, int height,
 			    int button_width, int button_height,
 			    ETableColArrow arrow)
@@ -220,6 +226,8 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 	int xthick, ythick;
 	int inner_x, inner_y;
 	int inner_width, inner_height;
+	GdkGC *gc;
+	char *text;
 
 	g_return_if_fail (drawable != NULL);
 	g_return_if_fail (ecol != NULL);
@@ -230,6 +238,18 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	g_return_if_fail (button_width > 0 && button_height > 0);
 
+	if (g_label == NULL) {
+		GtkWidget *button = gtk_button_new_with_label("Hi");
+		GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		g_label = GTK_BIN(button)->child;
+		gtk_container_add (GTK_CONTAINER (window), button);
+		gtk_widget_ensure_style (window);
+		gtk_widget_ensure_style (button);
+		gtk_widget_ensure_style (g_label);
+	}
+
+	gc = g_label->style->fg_gc[GTK_STATE_NORMAL];
+
 	xthick = style->klass->xthickness;
 	ythick = style->klass->ythickness;
 
@@ -285,9 +305,12 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 
 	/* Pixbuf or label */
 
+	text = e_utf8_to_gtk_string (widget, ecol->text);
+
 	if (ecol->is_pixbuf) {
 		int pwidth, pheight;
 		int clip_width, clip_height;
+		int xpos;
 		GdkPixmap *pixmap;
 
 		g_assert (ecol->pixbuf != NULL);
@@ -298,15 +321,34 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 		clip_width = MIN (pwidth, inner_width);
 		clip_height = MIN (pheight, inner_height);
 
+		xpos = inner_x;
+
+		if (inner_width - pwidth > 11) {
+			int rbearing;
+			int width;
+			int ypos;
+
+			gdk_string_extents (font, text, NULL, &rbearing, &width, NULL, NULL);
+			if (rbearing < inner_width - (pwidth + 1)) {
+				xpos = inner_x + (inner_width - width - (pwidth + 1)) / 2;
+			}
+
+			ypos = inner_y + (inner_height - font->ascent - font->descent) / 2 + font->ascent;
+
+			e_table_draw_elided_string (drawable, font, gc,
+						    xpos + pwidth + 1, ypos,
+						    text, inner_width - (xpos - inner_x), FALSE);
+		}
+
 		pixmap = make_composite_pixmap (drawable, gc,
 						ecol->pixbuf, &style->bg[state],
 						clip_width, clip_height,
-						inner_x,
+						xpos,
 						inner_y + (inner_height - clip_height) / 2);
 		if (pixmap) {
 			gdk_draw_pixmap (drawable, gc, pixmap,
 					 0, 0,
-					 inner_x,
+					 xpos,
 					 inner_y + (inner_height - clip_height) / 2,
 					 clip_width, clip_height);
 			gdk_pixmap_unref (pixmap);
@@ -318,8 +360,9 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 
 		e_table_draw_elided_string (drawable, font, gc,
 					    inner_x, ypos,
-					    ecol->text, inner_width, TRUE);
+					    text, inner_width, TRUE);
 	}
+	g_free (text);
 }
 
 /* Computes the length of a string that needs to be trimmed for elision */
