@@ -1,12 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-col-head.c: TableColHead implementation
+/* 
+ * e-table-header.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@ximian.com>
  *
- * (C) 1999 Ximian, Inc
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <string.h>
 #include <gtk/gtkobject.h>
@@ -20,12 +35,13 @@ enum {
 	ARG_0,
 	ARG_SORT_INFO,
 	ARG_WIDTH,
-	ARG_WIDTH_EXTRAS,
+	ARG_WIDTH_EXTRAS
 };
 
 enum {
 	STRUCTURE_CHANGE,
 	DIMENSION_CHANGE,
+	EXPANSION_CHANGE,
 	REQUEST_WIDTH,
 	LAST_SIGNAL
 };
@@ -220,11 +236,12 @@ eth_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 static void
 e_table_header_class_init (GtkObjectClass *object_class)
 {
+	ETableHeaderClass *klass = E_TABLE_HEADER_CLASS (object_class);
+
 	object_class->destroy = eth_destroy;
 	object_class->set_arg = eth_set_arg;
 	object_class->get_arg = eth_get_arg;
 
-
 	e_table_header_parent_class = (gtk_type_class (gtk_object_get_type ()));
 
 	gtk_object_add_arg_type ("ETableHeader::width", GTK_TYPE_DOUBLE, 
@@ -248,6 +265,13 @@ e_table_header_class_init (GtkObjectClass *object_class)
 				GTK_SIGNAL_OFFSET (ETableHeaderClass, dimension_change),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+	eth_signals [EXPANSION_CHANGE] = 
+		gtk_signal_new ("expansion_change", 
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (ETableHeaderClass, expansion_change),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
 	eth_signals [REQUEST_WIDTH] = 
 		gtk_signal_new ("request_width",
 				GTK_RUN_LAST,
@@ -257,6 +281,11 @@ e_table_header_class_init (GtkObjectClass *object_class)
 				GTK_TYPE_INT, 1, GTK_TYPE_INT);
 
 	E_OBJECT_CLASS_ADD_SIGNALS (object_class, eth_signals, LAST_SIGNAL);
+
+	klass->structure_change = NULL;
+	klass->dimension_change = NULL;
+	klass->expansion_change = NULL;
+	klass->request_width = NULL;
 }
 
 static void
@@ -540,6 +569,27 @@ e_table_header_total_width (ETableHeader *eth)
 	return total;
 }
 
+/**
+ * e_table_header_min_width:
+ * @eth: The ETableHeader to query
+ *
+ * Returns: the minimum number of pixels required by the @eth object.
+ **/
+int
+e_table_header_min_width (ETableHeader *eth)
+{
+	int total, i;
+	
+	g_return_val_if_fail (eth != NULL, 0);
+	g_return_val_if_fail (E_IS_TABLE_HEADER (eth), 0);
+
+	total = 0;
+	for (i = 0; i < eth->col_count; i++)
+		total += eth->columns [i]->min_width;
+
+	return total;
+}
+
 /**
  * e_table_header_move:
  * @eth: The ETableHeader to operate on.
@@ -570,7 +620,7 @@ e_table_header_move (ETableHeader *eth, int source_index, int target_index)
 	eth_do_insert (eth, target_index, old);
 	eth_update_offsets (eth);
 	
-	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [DIMENSION_CHANGE]);
+	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [DIMENSION_CHANGE], eth->width);
 	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [STRUCTURE_CHANGE]);
 }
 
@@ -662,6 +712,8 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 		for (i = idx + 1; i < eth->col_count; i++) {
 			eth->columns[i]->expansion = 0;
 		}
+
+		gtk_signal_emit (GTK_OBJECT (eth), eth_signals [EXPANSION_CHANGE]);
 		return;
 	}
 
@@ -671,6 +723,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 		for (i = idx; i < eth->col_count; i++) {
 			eth->columns[i]->expansion = 0;
 		}
+		gtk_signal_emit (GTK_OBJECT (eth), eth_signals [EXPANSION_CHANGE]);
 		return;
 	}
 
@@ -687,6 +740,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 		for (i = idx + 1; i < eth->col_count; i++) {
 			eth->columns[i]->expansion = 0;
 		}
+		gtk_signal_emit (GTK_OBJECT (eth), eth_signals [EXPANSION_CHANGE]);
 		return;
 	}
 	
@@ -711,6 +765,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 				eth->columns[i]->expansion = expansion / expandable_count;
 			}
 		}
+		gtk_signal_emit (GTK_OBJECT (eth), eth_signals [EXPANSION_CHANGE]);
 		return;
 	}
 
@@ -722,6 +777,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 			eth->columns[i]->expansion *= expansion / old_expansion;
 		}
 	}
+	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [EXPANSION_CHANGE]);
 }
 
 /**
@@ -767,6 +823,11 @@ eth_calc_widths (ETableHeader *eth)
 	int last_position = 0;
 	double next_position = 0;
 	int last_resizable = -1;
+	int *widths;
+	gboolean changed;
+
+	widths = g_new (int, eth->col_count);
+
 	/* - 1 to account for the last pixel border. */
 	extra = eth->width - 1;
 	expansion = 0;
@@ -775,21 +836,31 @@ eth_calc_widths (ETableHeader *eth)
 		if (eth->columns[i]->resizable && eth->columns[i]->expansion > 0)
 			last_resizable = i;
 		expansion += eth->columns[i]->resizable ? eth->columns[i]->expansion : 0;
-		eth->columns[i]->width = eth->columns[i]->min_width + eth->width_extras;
+		widths[i] = eth->columns[i]->min_width + eth->width_extras;
 	}
 	if (eth->sort_info)
 		extra -= e_table_sort_info_grouping_get_count(eth->sort_info) * GROUP_INDENT;
 	if (expansion != 0 && extra > 0) {
 		for (i = 0; i < last_resizable; i++) {
 			next_position += extra * (eth->columns[i]->resizable ? eth->columns[i]->expansion : 0)/expansion;
-			eth->columns[i]->width += next_position - last_position;
+			widths[i] += next_position - last_position;
 			last_position = next_position;
 		}
-		eth->columns[i]->width += extra - last_position;
+		widths[i] += extra - last_position;
 	}
 
+	changed = FALSE;
+
+	for (i = 0; i < eth->col_count; i++) {
+		if (eth->columns[i]->width != widths[i]) {
+			changed = TRUE;
+			eth->columns[i]->width = widths[i];
+		}
+	}
+	g_free (widths);
+	if (changed)
+		gtk_signal_emit (GTK_OBJECT (eth), eth_signals [DIMENSION_CHANGE], eth->width);
 	eth_update_offsets (eth);
-	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [DIMENSION_CHANGE]);
 }
 
 void
@@ -810,6 +881,7 @@ e_table_header_update_horizontal (ETableHeader *eth)
 		eth->columns[i]->expansion = 1;
 	}
 	enqueue(eth, -1, eth->nominal_width);
+	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [EXPANSION_CHANGE]);
 }
 
 GtkType
@@ -857,3 +929,26 @@ e_table_header_prioritized_column (ETableHeader *eth)
 	}
 	return best_model_col;
 }
+
+ETableCol *
+e_table_header_prioritized_column_selected (ETableHeader *eth, ETableColCheckFunc check_func, gpointer user_data)
+{
+	ETableCol *best_col = NULL;
+	int best_priority = G_MININT;
+	int i;
+	int count;
+
+	count = e_table_header_count (eth);
+	if (count == 0)
+		return NULL;
+	for (i = 1; i < count; i++) {
+		ETableCol *col = e_table_header_get_column (eth, i);
+		if (col) {
+			if ((best_col == NULL || col->priority > best_priority) && check_func (col, user_data)) {
+				best_priority = col->priority;
+				best_col = col;
+			}
+		}
+	}
+	return best_col;
+}
