@@ -1,15 +1,34 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-state.c: Savable state of a table.
+/* 
+ * e-table-state.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
  *   Chris Lahey <clahey@ximian.com>
- *   Miguel de Icaza (miguel@ximian.com)
  *
- * (C) 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <stdlib.h>
+#include <string.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkobject.h>
 #include <gnome-xml/parser.h>
@@ -30,7 +49,6 @@ etst_destroy (GtkObject *object)
 {
 	ETableState *etst = E_TABLE_STATE (object);
 
-	gtk_object_unref (GTK_OBJECT (etst->sort_info));
 	if (etst->columns) {
 		g_free (etst->columns);
 		etst->columns = NULL;
@@ -41,6 +59,11 @@ etst_destroy (GtkObject *object)
 		etst->expansions = NULL;
 	}
 	
+	if (etst->sort_info) {
+		gtk_object_unref (GTK_OBJECT (etst->sort_info));
+		etst->sort_info = NULL;
+	}
+	
 	GTK_OBJECT_CLASS (etst_parent_class)->destroy (object);
 }
 
@@ -62,7 +85,7 @@ etst_init (ETableState *state)
 	state->sort_info = e_table_sort_info_new();
 }
 
-E_MAKE_TYPE(e_table_state, "ETableState", ETableState, etst_class_init, etst_init, PARENT_TYPE);
+E_MAKE_TYPE(e_table_state, "ETableState", ETableState, etst_class_init, etst_init, PARENT_TYPE)
 
 ETableState *
 e_table_state_new (void)
@@ -162,10 +185,15 @@ e_table_state_save_to_file      (ETableState *state,
 				 const char          *filename)
 {
 	xmlDoc *doc;
-	doc = xmlNewDoc("1.0");
-	xmlDocSetRootElement(doc, e_table_state_save_to_node(state, NULL));
-	xmlSaveFile(filename, doc);
-	xmlFreeDoc(doc);
+	
+	if ((doc = xmlNewDoc ("1.0")) == NULL)
+		return;
+	
+	xmlDocSetRootElement (doc, e_table_state_save_to_node (state, NULL));
+	
+	e_xml_save_file (filename, doc);
+	
+	xmlFreeDoc (doc);
 }
 
 char *
@@ -239,5 +267,9 @@ e_table_state_duplicate (ETableState *state)
 	e_table_state_load_from_string (new_state, copy);
 	g_free (copy);
 
+	e_table_sort_info_set_can_group
+		(new_state->sort_info,
+		 e_table_sort_info_get_can_group (state->sort_info));
+
 	return new_state;
 }
